//notation: js file can only use this kind of comments
//since comments will cause error when use in webview.loadurl,
//comments will be remove by java use regexp
(function () {
  if (window.bridge && window.bridge.inited) {
    return;
  }

  var receiveMessageQueue = [];
  var messageHandlers = {};

  var responseCallbacks = {};
  var uniqueId = 1;

  // set default messageHandler  初始化默认的消息线程
  // init: 初始化桥接器，并指定一个消息处理函数作为参数。只能调用一次，重复调用会抛出错误。
  function init(messageHandler) {
    if (bridge._messageHandler) {
      throw new Error('bridge.init called twice');
    }
    // 项目中不使用此方式
    // _createQueueReadyIframe();
    // _createQueueReadyIframe4biz();
    bridge._messageHandler = messageHandler;
    var receivedMessages = receiveMessageQueue;
    receiveMessageQueue = null;
    for (var i = 0; i < receivedMessages.length; i++) {
      _dispatchMessageFromNative(receivedMessages[i]);
    }
    bridge.inited = true;
  }

  // 发送
  // 发送消息给原生客户端，可以附带数据，并可指定一个回调函数处理原生客户端的响应。
  function send(data, responseCallback) {
    _doSend('send', data, responseCallback);
  }

  // 注册线程 往数组里面添加值
  // 注册一个消息处理函数，用于处理从原生客户端发来的特定消息。
  function registerHandler(handlerName, handler) {
    messageHandlers[handlerName] = handler;
  }

  // 移除一个已注册的消息处理函数
  function removeHandler(handlerName, handler) {
    delete messageHandlers[handlerName];
  }

  // 调用线程
  // 前端调用原生客户端的特定消息处理函数，并可以传递数据和回调函数
  function callHandler(functionName, params, responseCallback) {
    // 如果方法不需要参数，只有回调函数，简化JS中的调用
    // 妙投业务中不会进入这个分支
    if (arguments.length == 2 && typeof params == 'function') {
      responseCallback = params;
      params = null;
    }
    _doSend(functionName, params, responseCallback);
  }

  // js 调用 native 场景下 responseCallback 为函数
  // native 调用 js 场景下 responseCallback 为字符串
  function _doSend(handlerName, message, responseCallback) {
    var callbackId;
    if (typeof responseCallback === 'string') {
      callbackId = responseCallback;
    } else if (responseCallback) {
      // 如果有回调函数，生成一个唯一的callbackId，妙投应进入此分支
      callbackId = 'cb_' + (uniqueId++) + '_' + new Date().getTime();
      responseCallbacks[callbackId] = responseCallback;
      // message.callbackId = callbackId;
    } else {
      callbackId = '';
    }
    try {
	  // 使用方括号访问对象的属性或方法
	  var fn = bridge[handlerName];
    } catch (e) {
      console.log(e);
    }
    if (typeof fn === 'function') {
      // 传递的消息是字符串，直接使用，否则序列化
      var passedMessage = typeof message === 'string' ? message : JSON.stringify(message);
      var responseData = fn.call(bridge, passedMessage, callbackId);
      // 如果没有回调函数，直接返回
      responseCallback = responseCallbacks[callbackId];
      if (!responseCallback) {
        return;
      }
      // 这里要注意一下
      // 如果有函数有返回值立即回调，比如现在用于测试的 nativeCallHandler 函数
      // 如果没有返回值，则等待 native 回调，所以这个时候还不能删除这个函数否则找不到了
      if (responseData) {
        var params = typeof responseData === 'string' ? JSON.parse(responseData) : responseData;
        responseCallback(params);
        delete responseCallbacks[callbackId];
      }
    }
  }

  // js 调用 native 场景下 responseId 不为空
  // native 调用 js 场景下 responseId 为空
  function _dispatchMessageFromNative(messageJSON) {
    setTimeout(function () {
      var message = JSON.parse(messageJSON);
      var responseCallback;
      if (message.responseId) {
        // js 调用 native 场景下进入此分支
        responseCallback = responseCallbacks[message.responseId];
        if (!responseCallback) {
          return;
        }
        if (message.responseData) {
          var params = typeof message.responseData === 'string' ? JSON.parse(message.responseData) : message.responseData;
          responseCallback(params);
        } else {
          responseCallback();
        }
        delete responseCallbacks[message.responseId];
      } else {
        // native 调用 js 场景下进入此分支
        if (message.callbackId) {
          var callbackResponseId = message.callbackId;
          responseCallback = function (responseData) {
            _doSend('response', responseData, callbackResponseId);
          };
        }

        var handler = bridge._messageHandler;
        if (message.handlerName) {
          handler = messageHandlers[message.handlerName];
        }
        // 查找指定handler
        try {
          // 前端仅接收对象
          var params = typeof message.data === 'string' ? JSON.parse(message.data) : message.data;
          handler(params, responseCallback);
        } catch (exception) {
          if (typeof console != 'undefined') {
            console.log('bridge: WARNING: javascript handler threw.', message, exception);
          }
        }
      }
    });
  }

  // 提供给 native 调用, receiveMessageQueue 在会在页面加载完后赋值为 null, 所以会执行 _dispatchMessageFromNative
  function _handleMessageFromNative(messageJSON) {
    if (receiveMessageQueue) {
      receiveMessageQueue.push(messageJSON);
    }
    _dispatchMessageFromNative(messageJSON);
  }

  bridge.init = init;
  bridge.doSend = send;
  bridge.registerHandler = registerHandler;
  bridge.callHandler = callHandler;
  bridge._handleMessageFromNative = _handleMessageFromNative;

  var readyEvent = document.createEvent('Events');
  var jobs = window.WVJBCallbacks || [];
  readyEvent.initEvent('WebViewJavascriptBridgeReady');
  readyEvent.bridge = bridge;
  window.WVJBCallbacks = [];
  jobs.forEach(function (job) {
    job(bridge);
  });
  document.dispatchEvent(readyEvent);

  // 初始化并且传入默认的处理器，当前端调用客户端不传递函数名时会使用此处理器
  // 虎嗅不支持这种方式，所以空实现
  bridge.init(function (message, responseCallback) {
        // nothing
  });
})();
